@@ -1,19 +1,15 @@
-
-#================================
 package SNMP::Effective::VarList;
-#================================
 
 use warnings;
 use strict;
 use SNMP;
 use Tie::Array;
 
-our @ISA = qw/Tie::StdArray/;
+our @ISA     = qw/Tie::StdArray/;
+our $VERSION = '1.05';
 
 
-sub PUSH { #==================================================================
-
-    ### init
+sub PUSH {
     my $self = shift;
     my @args = @_;
 
@@ -56,12 +52,10 @@ sub PUSH { #==================================================================
         }
     }
 
-    ### the end
     return $self->FETCHSIZE;
 }
 
-#=============================================================================
-1983;
+1;
 __END__
 
 =head1 NAME
@@ -70,7 +64,7 @@ SNMP::Effective::VarList - Helper module for SNMP::Effective
 
 =head1 VERSION
 
-This document refers to version 0.04 of SNMP::Effective::VarList.
+This document refers to version 1.05 of SNMP::Effective::VarList.
 
 =head1 DESCRIPTION
 
